/*
 * QtQueryListPipe.hpp
 *
 *  Created on: Oct 25, 2010
 *      Author: heber
 */

#ifndef QTQUERYLISTPIPE_HPP_
#define QTQUERYLISTPIPE_HPP_

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "Parameters/Parameter.hpp"
#include "QtQueryPipe.hpp"

// Qt moc needs these because it generates its own .cpp file from QtQueryListPipe.hpp.
#include <Qt/qboxlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qlineedit.h>
#include <Qt/qlistwidget.h>
#include <Qt/qpushbutton.h>

#include <boost/lexical_cast.hpp>

using boost::lexical_cast;
using boost::bad_lexical_cast;


template<typename T> QtQueryListPipe<T>::QtQueryListPipe(Parameter<std::vector<T> > &_content, QtDialog *_dialog, QLineEdit *_inputBox, QListWidget *_inputList, QPushButton *_AddButton, QPushButton *_RemoveButton) :
  content(_content),
  dialog(_dialog),
  inputBox(_inputBox),
  inputList(_inputList),
  AddButton(_AddButton),
  RemoveButton(_RemoveButton)
{}

template<typename T> QtQueryListPipe<T>::~QtQueryListPipe()
{}

template<typename T> void QtQueryListPipe<T>::elementEntered(const QString& t)
{
  AddButton->setEnabled(t.length() > 0);
}

template<typename T> void QtQueryListPipe<T>::elementSelected()
{
  if (inputList->selectedItems().empty())
    RemoveButton->setEnabled(false);
  else
    RemoveButton->setEnabled(true);
}

template<typename T> void QtQueryListPipe<T>::addElement() {
  // type-check
  std::string text = inputBox->text().toStdString();
  T value;
  try {
    value = boost::lexical_cast<T>(text);
  } catch (boost::bad_lexical_cast&) {
    return;
  };
  // add item to both
  inputList->addItem(QString(toString(value).c_str()));
  addValue(value);
}

template<typename T> void QtQueryListPipe<T>::addValue(T item) {
  std::vector<T> temp = content.get();
  temp.push_back(item);
  content.set(temp);

  dialog->update();
}

template<typename T> void QtQueryListPipe<T>::removeElement() {
  QList<QListWidgetItem *> items = inputList->selectedItems();
  for (QList<QListWidgetItem *>::iterator iter = items.begin(); !items.empty(); iter = items.begin()) {
    // obtain which position item has (by making it current item)
    inputList->setCurrentItem(*iter);
    // remove
    QtQueryListPipe<T>::removeRow(inputList->currentRow()); // template parameters needs to be known, such that compiler knows which to call
    inputList->takeItem(inputList->currentRow());
    items.erase(iter);
  }
}

template<typename T> void QtQueryListPipe<T>::removeRow(int row) {
  int counter = 0;
  std::vector<T> temp = content.get();
  typename std::vector<T>::iterator iter = temp.begin();
  for (; iter != temp.end(); ++iter)
    if (counter++ == row)
      break;
  if (iter != temp.end())
    temp.erase(iter);
  content.set(temp);
}

class StringsQtQueryPipe : public QtQueryListPipe<std::string>
{
  Q_OBJECT
public:
  StringsQtQueryPipe(Parameter<std::vector<std::string> > &_content, QtDialog *_dialog, QLineEdit *_inputBox, QListWidget *_inputList, QPushButton *_AddButton, QPushButton *_RemoveButton) :
    QtQueryListPipe<std::string>(_content, _dialog, _inputBox, _inputList, _AddButton, _RemoveButton) { }

public slots:
  virtual void elementEntered(const QString& t)
  {
    QtQueryListPipe<std::string>::elementEntered(t);
  }
  virtual void elementSelected()
  {
    QtQueryListPipe<std::string>::elementSelected();
  }
  virtual void addElement()
  {
    QtQueryListPipe<std::string>::addElement();
  }
  virtual void removeElement()
  {
    QtQueryListPipe<std::string>::removeElement();
  }
};


#endif /*  QTQUERYLISTPIPE_HPP_  */
